
#include <stdio.h>
#include <math.h>
#include <unistd.h>
#include <stdlib.h>
#include <string.h>
#include <sys/time.h>
#include "pngoutput.h"
#include "tools.h"
#include "leetft.h"
#include "videotools.h"
#include "speedy.h"

static double intensity_to_voltage_srgb( double val )
{
    /* Handle invalid values before doing a gamma transform. */
    if( val < 0.0 ) return 0.0;
    if( val > 1.0 ) return 1.0;

    /* sRGB uses an exponent of 2.4 in its nonstandard curve, but it
     *      * still advertises a display gamma of 2.2.
     *           */
    if( val <= 0.0031308 ) {
        return 12.92 * val;
    }
    return ( ( 1.055 * pow( val, 1.0 / 2.4 ) ) - 0.055 );
}

static int crawlpage = 0;

void output_next_crawlpage( unsigned char *data, int width, int height, int stride )
{
    char filename[ 1024 ];
    pngoutput_t *pngout;

    sprintf( filename, "/tmp/crawl%08d.png", crawlpage++ );
    pngout = pngoutput_new( filename, width, height, 1, 0.45, 0 );
    if( pngout ) {
        unsigned char tempscanline[ 4096 ];
        int i;

        for( i = 0; i < height; i++ ) {
            argb32_to_rgba32_scanline( tempscanline, data + (i * stride), width );
            pngoutput_scanline( pngout, tempscanline );
        }
        pngoutput_delete( pngout );
    }
}

int main( int argc, char **argv )
{
    FILE *infile;
    int width, height, stride;
    ft_font_t *font;
    ft_string_t *fts;
    double pixel_aspect;
    int pos = 0;
    int crawlwidth;
    unsigned char *renderdata;

    setup_speedy_calls( 1 );

    crawlwidth = 1024;
    width = crawlwidth * 2;
    height = 60;
    stride = width * 4;

    renderdata = (unsigned char *) malloc( stride * height );
    if( !renderdata ) {
        fprintf( stderr, "you suck mcbain\n" );
        return 1;
    }
    memset( renderdata, 0, stride * height );

    pixel_aspect = 1.0;
    font = ft_font_new( "Verdana.ttf", 30, pixel_aspect );
    if( !font ) {
        fprintf( stderr, "can't open font?\n" );
        return 1;
    }

    fts = ft_string_new( font );

    infile = fopen( argv[ 1 ], "r" );
    if( !infile ) {
        fprintf( stderr, "can't open %s\n", argv[ 1 ] );
        return 1;
    }

    for(;;) {
        char curline[ 1024 ];
        fgets( curline, sizeof( curline ) - 1, infile );
        if( feof( infile ) ) break;

        ft_string_set_text( fts, curline, 0 );

        composite_alphamask_to_packed4444( renderdata, width, height, stride,
                                           ft_string_get_buffer( fts ), ft_string_get_width( fts ),
                                           ft_string_get_height( fts ), ft_string_get_stride( fts ),
                                           250, 235, 149, pos, 0 );
        pos += ft_string_get_width( fts ) + 30;
        if( pos > crawlwidth ) {
            int i;
            output_next_crawlpage( renderdata, crawlwidth, height, stride );
            for( i = 0; i < height; i++ ) {
                memcpy( renderdata + (i * stride),
                        renderdata + (i * stride) + (crawlwidth * 4),
                        crawlwidth * 4 );
                memset( renderdata + (i * stride) + (crawlwidth * 4), 0, crawlwidth * 4 );
            }
            pos -= crawlwidth;
        }
    }
    output_next_crawlpage( renderdata, pos, height, stride );

    fclose( infile );
    return 0;
}

